@@ -16,10 +16,8 @@ built on thousands of lines of code in other libraries. Those libraries had
 already been deployed, used, and tested, and were known to be
 "production ready."
 
-TurboGears2 returns to that philosophy. It is built on Pylons, but it brings
-a best-of-breed approach to Pylons. TurboGears 2 is commited to the following
-Python components and libraries, which are backwards compatable with
-TurboGears 1.1:
+TurboGears 2 is committed to the following Python components and libraries,
+which are backwards compatible with TurboGears 1.1:
 
     * Models: SQLAlchemy
     * Template engines: Genshi
@@ -36,33 +34,34 @@ The zen of TurboGears is::
 Mark Ramm described the relationship between TurboGears and Pylons this way
 "TurboGears 2 is to Pylons as Ubuntu is to Debian."
 
-In other words we're focused on user experience, and creating a novice-friendly
-environment. We ship a smaller subset of components, and thus are better able
+While TurboGears2 is not based on Pylons anymore it's still focused on user experience,
+and creating a novice-friendly environment.
+We ship a smaller subset of components, and thus are better able
 to focus, test, and document things so that new users have the best possible
 experience.
 
-Meanwhile Pylons provides the power and flexibility of the underlying core.
-
-And like Ubuntu, we don't intend to hide that power and flexibility from
-advanced users, but we know that they want things set up to just work too.
-
 Sensible defaults encourage code re-use within TurboGears because
 they make it possible for a group of TurboGears components to share
 assumptions about how things will work.
 
 """
 
-from tg.request_local import app_globals, request, response, tmpl_context, session, cache, translator
-from tg.configuration import config, AppConfig
+from tg.request_local import (app_globals, request, response, tmpl_context,
+                              session, cache, translator)
+from tg.configuration import config, milestones
+from tg.configuration.app_config import AppConfig
+from tg.configurator import (Configurator, ApplicationConfigurator,
+                             MinimalApplicationConfigurator,
+                             FullStackApplicationConfigurator)
 from tg.wsgiapp import TGApp
 from tg.controllers import TGController, RestController, redirect, url, lurl, abort
 from tg.release import version
 from tg.decorators import (validate, expose, override_template, use_custom_format,
-                           require, with_engine, cached)
+                           require, with_engine, cached, decode_params)
 
-from tg.flash import flash, get_flash, get_status
+from tg.flash import flash
 from tg.jsonify import encode as json_encode
-from tg.controllers.util import use_wsgi_app
+from tg.controllers.util import *
 from tg.controllers.dispatcher import dispatched_controller
 from tg.render import render as render_template
 from tg.configuration.hooks import hooks
@@ -72,8 +71,9 @@ from tg.request_local import Request, Response
 __version__ = version
 
 __all__ = ['__version__',
-    'app_globals', 'expose', 'override_template', 'request',
-    'require', 'response', 'session', 'TGApp', 'TGController', 'tmpl_context',
-    'use_wsgi_app', 'validate', 'i18n','json_encode', 'cache', 'url', 'lurl',
-    'dispatched_controller', 'use_custom_format', 'with_engine', 'render_template',
-    'Request', 'Response', 'cached']
+           'app_globals', 'expose', 'override_template', 'request',
+           'require', 'response', 'session', 'TGApp', 'TGController', 'tmpl_context',
+           'use_wsgi_app', 'validate', 'i18n', 'json_encode', 'cache', 'url', 'lurl',
+           'dispatched_controller', 'use_custom_format', 'with_engine', 'render_template',
+           'Request', 'Response', 'cached', 'decode_params', 'milestones',
+           'MinimalApplicationConfigurator', 'FullStackApplicationConfigurator']
