@@ -7,6 +7,8 @@ else: # pragma: no cover
 
 # True if we are running on Python 3.
 PY3 = sys.version_info[0] == 3
+PY2 = sys.version_info[0] == 2
+
 
 if PY3: # pragma: no cover
     string_type = str
@@ -22,7 +24,7 @@ if PY3: # pragma: no cover
 
     def bytes_(s):
         return str(s).encode('ascii', 'strict')
-else:
+else: # pragma: no cover
     string_type = basestring
     unicode_text = unicode
     byte_string = str
@@ -37,24 +39,28 @@ else:
     def bytes_(s):
         return str(s)
 
+
 def im_func(f):
     if PY3: # pragma: no cover
         return getattr(f, '__func__', None)
-    else:
+    else: # pragma: no cover
         return getattr(f, 'im_func', None)
 
+
 def default_im_func(f):
     if PY3: # pragma: no cover
         return getattr(f, '__func__', f)
-    else:
+    else: # pragma: no cover
         return getattr(f, 'im_func', f)
 
+
 def im_self(f):
     if PY3: # pragma: no cover
         return getattr(f, '__self__', None)
-    else:
+    else: # pragma: no cover
         return getattr(f, 'im_self', None)
 
+
 def im_class(f):
     if PY3: # pragma: no cover
         self = im_self(f)
@@ -62,13 +68,15 @@ def im_class(f):
             return self.__class__
         else:
             return None
-    else:
+    else: # pragma: no cover
         return getattr(f, 'im_class', None)
 
+
 def with_metaclass(meta, base=object):
     """Create a base class with a metaclass."""
     return meta("NewBase", (base,), {})
 
+
 if PY3: # pragma: no cover
     import builtins
     exec_ = getattr(builtins, "exec")
@@ -94,3 +102,43 @@ else: # pragma: no cover
     exec_("""def reraise(tp, value, tb=None):
     raise tp, value, tb
 """)
+
+
+try:  # pragma: no cover
+    from importlib import import_module
+except ImportError:  # pragma: no cover
+    # Compatibility module for python2.6
+    import sys
+
+    def _resolve_name(name, package, level):
+        """Return the absolute name of the module to be imported."""
+        if not hasattr(package, 'rindex'):
+            raise ValueError("'package' not set to a string")
+        dot = len(package)
+        for x in xrange(level, 1, -1):
+            try:
+                dot = package.rindex('.', 0, dot)
+            except ValueError:
+                raise ValueError("attempted relative import beyond top-level "
+                                 "package")
+        return "%s.%s" % (package[:dot], name)
+
+    def import_module(name, package=None):
+        """Import a module.
+
+        The 'package' argument is required when performing a relative import. It
+        specifies the package to use as the anchor point from which to resolve the
+        relative import to an absolute import.
+
+        """
+        if name.startswith('.'):
+            if not package:
+                raise TypeError("relative imports require the 'package' argument")
+            level = 0
+            for character in name:
+                if character != '.':
+                    break
+                level += 1
+            name = _resolve_name(name[level:], package, level)
+        __import__(name)
+        return sys.modules[name]
\ No newline at end of file
