@@ -6,6 +6,7 @@ and removes the dependency from repoze.what.
 """
 import sys, logging, re
 from tg.support.converters import asbool
+from ..utils import get_partial_dict
 from zope.interface import implementer
 from repoze.who.middleware import PluggableAuthenticationMiddleware
 from repoze.who.interfaces import IIdentifier, IAuthenticator, IChallenger
@@ -13,8 +14,6 @@ from repoze.who.classifiers import default_challenge_decider, default_request_cl
 from repoze.who.config import _LEVELS
 from webob.exc import HTTPUnauthorized
 
-from tg.configuration.auth.metadata import _AuthMetadataProvider
-
 log = logging.getLogger(__name__)
 
 @implementer(IIdentifier, IAuthenticator, IChallenger)
@@ -92,14 +91,20 @@ class _AuthenticationForgerMiddleware(PluggableAuthenticationMiddleware):
             classifier, challenge_decider, log_stream, log_level,
             'repoze.who.testutil.userid')
 
-def setup_auth(app, authmetadata,
-              form_plugin=None, form_identifies=True,
-              cookie_secret='secret', cookie_name='authtkt',
-              login_url='/login', login_handler='/login_handler',
-              post_login_url=None, logout_handler='/logout_handler',
-              post_logout_url=None, login_counter_name=None,
-              cookie_timeout=None, cookie_reissue_time=None,
-              **who_args):
+
+def turbogears_challenge_decider(environ, status, headers):
+    if 'tg.skip_auth_challenge' in environ:
+        return None
+
+    return default_challenge_decider(environ, status, headers)
+
+
+def setup_auth(app, form_plugin=None, form_identifies=True,
+               cookie_secret='secret', cookie_name='authtkt',
+               login_url='/login', login_handler='/login_handler',
+               post_login_url=None, logout_handler='/logout_handler',
+               post_logout_url=None, login_counter_name=None,
+               **who_args):
     """
     Sets :mod:`repoze.who` up with the provided authenticators and
     options to create FriendlyFormPlugin/FastFormPlugin.
@@ -109,17 +114,26 @@ def setup_auth(app, authmetadata,
     with repoze.who and repoze.what.
     """
     if 'charset' in who_args: #pragma: no cover
-        log.warn('charset argument in authentication setup is ignored')
+        log.warning('charset argument in authentication setup is ignored')
         who_args.pop('charset')
 
     # If no identifiers are provided in repoze setup arguments
     # then create a default one using AuthTktCookiePlugin.
-    if 'identifiers' not in who_args:
+    identifiers = who_args.setdefault('identifiers', [('default', None)])
+    try:
+        default_identifier_index = identifiers.index(('default', None))
+    except ValueError:
+        pass
+    else:
+        authtkt_options = get_partial_dict('authtkt', who_args,
+                                           ignore_missing=True,
+                                           pop_keys=True)
+
+        # default identifier included.
         from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
-        cookie = AuthTktCookiePlugin(cookie_secret, cookie_name,
-                                     timeout=cookie_timeout,
-                                     reissue_time=cookie_reissue_time)
-        who_args['identifiers'] = [('cookie', cookie)]
+        cookie = AuthTktCookiePlugin(cookie_secret, cookie_name, **authtkt_options)
+        who_args['identifiers'][default_identifier_index] = ('cookie', cookie)
+        who_args.setdefault('authenticators', [])
         who_args['authenticators'].insert(0, ('cookie', cookie))
 
     # If no form plugin is provided then create a default
@@ -159,21 +173,17 @@ def setup_auth(app, authmetadata,
         log_level = _LEVELS[log_level.lower()]
     who_args['log_level'] = log_level
 
-    # Setting up the metadata provider for the user informations
+    # Setting up the metadata provider for the user information
     if 'mdproviders' not in who_args:
         who_args['mdproviders'] = []
 
-    if authmetadata:
-        authmd = _AuthMetadataProvider(authmetadata)
-        who_args['mdproviders'].append(('authmd', authmd))
-
     # Set up default classifier
     if 'classifier' not in who_args:
         who_args['classifier'] = default_request_classifier
 
     # Set up default challenger decider
     if 'challenge_decider' not in who_args:
-        who_args['challenge_decider'] = default_challenge_decider
+        who_args['challenge_decider'] = turbogears_challenge_decider
 
     skip_authn = who_args.pop('skip_authentication', False)
     if asbool(skip_authn):
