@@ -5,18 +5,38 @@ Utilities for TurboGears hooks management.
 Provides a consistent API to register and execute hooks.
 
 """
-from tg.configuration.utils import TGConfigError
-from tg.configuration.milestones import config_ready, renderers_ready, environment_loaded
-from tg.decorators import Decoration
-from tg._compat import default_im_func
-import tg
+import atexit
+from .utils import TGConfigError
+from .milestones import config_ready, renderers_ready
+from ..controllers.decoration import Decoration
+from .._compat import default_im_func
+
 
 from logging import getLogger
 log = getLogger(__name__)
 
 
-class _TGHooks(object):
+class HooksNamespace(object):
     """Manages hooks registrations and notifications"""
+    def __init__(self):
+        self._hooks = dict()
+        atexit.register(self._atexit)
+
+    def _clear(self):
+        self._hooks.clear()
+
+    def _atexit(self):
+        for func in self._hooks.get('shutdown', tuple()):
+            func()
+
+    def _call_handler(self, hook_name, trap_exceptions, func, args, kwargs):
+        try:
+            return func(*args, **kwargs)
+        except:
+            if trap_exceptions is True:
+                log.exception('Trapped Exception while handling %s -> %s', hook_name, func)
+            else:
+                raise
 
     def register(self, hook_name, func, controller=None):
         """Registers a TurboGears hook.
@@ -32,58 +52,39 @@ class _TGHooks(object):
             tg.hooks.register('startup', startup_function)
 
         """
-        if hook_name in ('startup', 'shutdown') and controller is not None:
+        if controller is not None and hook_name in ('startup', 'shutdown',
+                                                    'initialized_config',
+                                                    'before_wsgi_middlewares',
+                                                    'after_wsgi_middlewares'):
             raise TGConfigError('Startup and Shutdown hooks cannot be registered on controllers')
 
         if hook_name == 'controller_wrapper':
-            raise TGConfigError('tg.hooks.wrap_controller must be used to register wrappers')
+            raise ValueError('dispatch component must be used to register controller wrappers')
 
         if controller is None:
-            config_ready.register(_ApplicationHookRegistration(hook_name, func))
+            config_ready.register(_ApplicationHookRegistration(self, hook_name, func))
         else:
             controller = default_im_func(controller)
             renderers_ready.register(_ControllerHookRegistration(controller, hook_name, func))
 
-    def wrap_controller(self, func, controller=None):
-        """Registers a TurboGears controller wrapper.
-
-        Controller Wrappers are much like a **decorator** applied to
-        every controller.
-        They receive :class:`tg.configuration.AppConfig` instance
-        as an argument and the next handler in chain and are expected
-        to return a new handler that performs whatever it requires
-        and then calls the next handler.
-
-        A simple example for a controller wrapper is a simple logging wrapper::
-
-            def controller_wrapper(app_config, caller):
-                def call(*args, **kw):
-                    try:
-                        print 'Before handler!'
-                        return caller(*args, **kw)
-                    finally:
-                        print 'After Handler!'
-                return call
-
-            tg.hooks.wrap_controller(controller_wrapper)
-
-        It is also possible to register wrappers for a specific controller::
-
-            tg.hooks.wrap_controller(controller_wrapper, controller=RootController.index)
+    def disconnect(self, hook_name, func, controller=None):
+        """Disconnect an hook.
 
+        The registered function is removed from the hook notification list.
         """
-        if environment_loaded.reached:
-            raise TGConfigError('Controller wrappers can be registered only at '
-                                'configuration time.')
-
         if controller is None:
-            environment_loaded.register(_ApplicationHookRegistration('controller_wrapper', func))
+            registrations = self._hooks.get(hook_name, [])
         else:
-            controller = default_im_func(controller)
-            registration = _ControllerHookRegistration(controller, 'controller_wrapper', func)
-            renderers_ready.register(registration)
+            deco = Decoration.get_decoration(controller)
+            registrations = deco.hooks.get(hook_name, [])
 
-    def notify(self, hook_name, args=None, kwargs=None, controller=None, context_config=None):
+        try:
+            registrations.remove(func)
+        except ValueError:
+            pass
+
+    def notify(self, hook_name, args=None, kwargs=None, controller=None,
+               trap_exceptions=False):
         """Notifies a TurboGears hook.
 
         Each function registered for the given hook will be executed,
@@ -100,27 +101,24 @@ class _TGHooks(object):
                             controller=RootController.index)
 
         """
-        if context_config is None: #pragma: no cover
-            context_config = tg.config._current_obj()
-
         args = args or []
         kwargs = kwargs or {}
 
         try:
-            syswide_hooks = context_config['hooks'][hook_name]
+            syswide_hooks = self._hooks[hook_name]
         except KeyError:  # pragma: no cover
             pass
         else:
             for func in syswide_hooks:
-                func(*args, **kwargs)
+                self._call_handler(hook_name, trap_exceptions, func, args, kwargs)
 
         if controller is not None:
             controller = default_im_func(controller)
             deco = Decoration.get_decoration(controller)
             for func in deco.hooks.get(hook_name, []):
-                func(*args, **kwargs)
+                self._call_handler(hook_name, trap_exceptions, func, args, kwargs)
 
-    def notify_with_value(self, hook_name, value, controller=None, context_config=None):
+    def notify_with_value(self, hook_name, value, controller=None):
         """Notifies a TurboGears hook which is expected to return a value.
 
         hooks with values are expected to accept an input value an return
@@ -133,11 +131,8 @@ class _TGHooks(object):
             app = tg.hooks.notify_with_value('before_config', app)
 
         """
-        if context_config is None:   # pragma: no cover
-            context_config = tg.config._current_obj()
-
         try:
-            syswide_hooks = context_config['hooks'][hook_name]
+            syswide_hooks = self._hooks[hook_name]
         except KeyError:  # pragma: no cover
             pass
         else:
@@ -154,23 +149,19 @@ class _TGHooks(object):
 
 
 class _ApplicationHookRegistration(object):
-    def __init__(self, hook_name, func):
+    def __init__(self, hooks_namespace, hook_name, func):
         self.hook_name = hook_name
         self.func = func
+        self.hooks_namespace = hooks_namespace
+
+    def __repr__(self):
+        return '<ApplicationHookRegistration: %r %r>' % (self.hook_name, self.func)
 
     def __call__(self):
         log.debug("Registering %s for application wide hook %s",
                   self.func, self.hook_name)
-
-        config = tg.config._current_obj()
-        if self.hook_name == 'startup':
-            config['call_on_startup'].append(self.func)
-        elif self.hook_name == 'shutdown':
-            config['call_on_shutdown'].append(self.func)
-        elif self.hook_name == 'controller_wrapper':
-            config['controller_wrappers'].append(self.func)
-        else:
-            config['hooks'].setdefault(self.hook_name, []).append(self.func)
+        hooks = self.hooks_namespace._hooks
+        hooks.setdefault(self.hook_name, []).append(self.func)
 
 
 class _ControllerHookRegistration(object):
@@ -179,16 +170,17 @@ class _ControllerHookRegistration(object):
         self.hook_name = hook_name
         self.func = func
 
+    def __repr__(self):
+        return '<ControllerHookRegistration: %r %r>' % (self.hook_name, self.func)
+
     def __call__(self):
         log.debug("Registering %s for hook %s on controller %s",
                   self.func, self.hook_name, self.controller)
-
-        if self.hook_name == 'controller_wrapper':
-            deco = Decoration.get_decoration(self.controller)
-            deco._register_controller_wrapper(self.func)
-        else:
-            deco = Decoration.get_decoration(self.controller)
-            deco._register_hook(self.hook_name, self.func)
+        deco = Decoration.get_decoration(self.controller)
+        deco._register_hook(self.hook_name, self.func)
 
 
-hooks = _TGHooks()
+class _TGGlobalHooksNamespace(HooksNamespace):
+    pass
+
+hooks = _TGGlobalHooksNamespace()
