@@ -6,23 +6,21 @@ decorators to effect a rendered page.
 """
 
 import inspect, operator
+import warnings
 import tg
 from tg.controllers.util import abort
 from tg.predicates import NotAuthorizedError, not_anonymous
 
-from crank.util import (get_params_with_argspec,
-                        remove_argspec_params_from_params)
+from crank.util import get_params_with_argspec, flatten_arguments
 
 from tg.flash import flash
 from tg.jsonify import JsonEncodeError
 from tg.render import render as tg_render
-from tg.util import Bunch
-from tg.validation import (_navigate_tw2form_children, _FormEncodeSchema,
+from tg.validation import (_navigate_tw2form_children,
                            _Tw2ValidationError, validation_errors,
-                           _FormEncodeValidator, TGValidationError)
+                           TGValidationError, _ValidationStatus)
 
 from tg._compat import unicode_text, with_metaclass, im_self, url2pathname, default_im_func
-from tg.configuration.hooks import hooks
 from functools import partial
 
 strip_string = operator.methodcaller('strip')
@@ -32,7 +30,7 @@ class _DecoratedControllerMeta(type):
     def __init__(cls, name, bases, attrs):
         super(_DecoratedControllerMeta, cls).__init__(name, bases, attrs)
         for name, value in attrs.items():
-            #Inherit decorations for methods exposed with inherit=True
+            # Inherit decorations for methods exposed with inherit=True
             if hasattr(value, 'decoration') and value.decoration.inherit:
                 for pcls in reversed(bases):
                     parent_method = getattr(pcls, name, None)
@@ -52,7 +50,7 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         if method and inspect.ismethod(method) and hasattr(method, 'decoration'):
             return method.decoration.exposed
 
-    def _call(self, controller, params, remainder=None, context=None):
+    def _call(self, action, params, remainder=None, context=None):
         """Run the controller with the given parameters.
 
         _call is called by _perform_call in CoreDispatcher.
@@ -79,12 +77,13 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
             #compatibility with old code that didn't pass request locals explicitly
             context = tg.request.environ['tg.locals']
 
+        hooks = tg.hooks
         context_config = tg.config._current_obj()
-        self._initialize_validation_context(context)
+        context.request._fast_setattr('validation', _ValidationStatus())
 
-        #This is necessary to prevent spurious Content Type header which would
-        #cause problems to paste.response.replace_header calls and cause
-        #responses wihout content type to get out with a wrong content type
+        # This is necessary to prevent spurious Content Type header which would
+        # cause problems to paste.response.replace_header calls and cause
+        # responses without content type to get out with a wrong content type
         resp_headers = context.response.headers
         if not resp_headers.get('Content-Type'):
             resp_headers.pop('Content-Type', None)
@@ -94,44 +93,52 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         else:
             remainder = tuple()
 
-        hooks.notify('before_validate', args=(remainder, params),
-                     controller=controller, context_config=context_config)
+        hooks.notify('before_validate', args=(remainder, params), controller=action)
+
+        validate_params = get_params_with_argspec(action, params, remainder)
+        context.request.args_params = validate_params  # Update args_params with positional args
 
         try:
-            validate_params = get_params_with_argspec(controller, params, remainder)
-
-            # Validate user input
-            params = self._perform_validate(controller, validate_params)
-            context.request.validation['values'] = params
-
-            params, remainder = remove_argspec_params_from_params(controller, params, remainder)
-            bound_controller_callable = controller
+            params = self._perform_validate(action, validate_params, context)
         except validation_errors as inv:
-            instance, controller = self._process_validation_errors(controller,
-                                                                   remainder, params,
-                                                                   inv, context=context)
-            bound_controller_callable = partial(controller, instance)
+            instance, error_handler, chain_validation = self._process_validation_errors(
+                action, remainder, params, inv, context=context
+            )
+            while chain_validation:
+                # The validation asked for chained validation,
+                # go on and validate the error_handler too.
+                try:
+                    params = self._perform_validate(error_handler, validate_params, context)
+                except validation_errors as inv:
+                    instance, error_handler, chain_validation = self._process_validation_errors(
+                        error_handler, remainder, params, inv, context=context
+                    )
+                else:
+                    chain_validation = False
+            action = error_handler
+            bound_controller_callable = partial(error_handler, instance)
+        else:
+            bound_controller_callable = action
+            context.request.validation.values = params
+            remainder, params = flatten_arguments(action, params, remainder)
 
-        hooks.notify('before_call', args=(remainder, params),
-                     controller=controller, context_config=context_config)
+        hooks.notify('before_call', args=(remainder, params), controller=action)
 
         # call controller method with applied wrappers
-        controller_caller = controller.decoration.controller_caller
+        controller_caller = action.decoration.controller_caller
         output = controller_caller(context_config, bound_controller_callable, remainder, params)
 
         # Render template
-        hooks.notify('before_render', args=(remainder, params, output),
-                     controller=controller, context_config=context_config)
+        hooks.notify('before_render', args=(remainder, params, output), controller=action)
 
-        response = self._render_response(context, controller, output)
+        response = self._render_response(context, action, output)
 
-        hooks.notify('after_render', args=(response,),
-                     controller=controller, context_config=context_config)
+        hooks.notify('after_render', args=(response,), controller=action)
 
         return response['response']
 
     @classmethod
-    def _perform_validate(cls, controller, params):
+    def _perform_validate(cls, controller, params, context=None):
         """Run validation for the controller with the given parameters.
 
         Validation is stored on the "validation" attribute of the controller's
@@ -152,69 +159,24 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         be FormEncode Invalid objects.
 
         """
+        if context is None:  # pragma: no cover
+            warnings.warn("Calling DecoratedController._perform_validate without a Context is now deprecated."
+                          " Please provide the context argument when calling it.",
+                          DeprecationWarning, stacklevel=2)
+            context = tg.request.environ['tg.locals']
 
-        validation = getattr(controller.decoration, 'validation', None)
-
-        if validation is None:
+        validations = controller.decoration.validations
+        if not validations:
             return params
 
-        # An object used by FormEncode to get translator function
-        formencode_state = type('state', (), {'_': staticmethod(tg.i18n._formencode_gettext)})
+        req = context.request
+        validation_status = req.validation
 
-        #Initialize new_params -- if it never gets updated just return params
-        new_params = {}
-
-        # The validator may be a dictionary, a FormEncode Schema object, or any
-        # object with a "validate" method.
-        if isinstance(validation.validators, dict):
-            # TG developers can pass in a dict of param names and FormEncode
-            # validators.  They are applied one by one and builds up a new set
-            # of validated params.
-
-            errors = {}
-            for field, validator in validation.validators.items():
-                try:
-                    if isinstance(validator, _FormEncodeValidator):
-                        new_params[field] = validator.to_python(params.get(field),
-                                                                formencode_state)
-                    else:
-                        new_params[field] = validator.to_python(params.get(field))
-                # catch individual validation errors into the errors dictionary
-                except validation_errors as inv:
-                    errors[field] = inv
-
-            # Parameters that don't have validators are returned verbatim
-            for param, param_value in params.items():
-                if not param in new_params:
-                    new_params[param] = param_value
-
-            # If there are errors, create a compound validation error based on
-            # the errors dictionary, and raise it as an exception
-            if errors:
-                raise TGValidationError(TGValidationError.make_compound_message(errors),
-                                        value=params,
-                                        error_dict=errors)
-
-        elif isinstance(validation.validators, _FormEncodeSchema):
-            # A FormEncode Schema object - to_python converts the incoming
-            # parameters to sanitized Python values
-            new_params = validation.validators.to_python(params, formencode_state)
-
-        elif (hasattr(validation.validators, 'validate')
-              and getattr(validation, 'needs_controller', False)):
-            # An object with a "validate" method - call it with the parameters
-            new_params = validation.validators.validate(
-                controller, params, formencode_state)
-
-        elif hasattr(validation.validators, 'validate'):
-            # An object with a "validate" method - call it with the parameters
-            new_params = validation.validators.validate(params, formencode_state)
-
-        # Theoretically this should not happen...
-        # if new_params is None:
-        #     return params
-
-        return new_params
+        validated_params = params
+        for validation_intent in validations:
+            validation_status.intent = validation_intent
+            validated_params = validation_intent.check(controller, validated_params)
+        return validated_params
 
     def _render_response(self, tgl, controller, response):
         """
@@ -237,29 +199,29 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         req = tgl.request
         resp = tgl.response
 
-        (content_type, engine_name, template_name, exclude_names, render_params
-            ) = controller.decoration.lookup_template_engine(tgl)
+        (engine_content_type, engine_name, template_name,
+         exclude_names, render_params) = controller.decoration.lookup_template_engine(tgl)
 
-        result = dict(response=response, content_type=content_type,
+        result = dict(response=response, content_type=engine_content_type,
                       engine_name=engine_name, template_name=template_name)
 
-        if content_type is not None:
-            resp.headers['Content-Type'] = content_type
+        if resp.content_type is None and engine_content_type is not None:
+            # User didn't set a specific content type during controller
+            # and template engine has a suggested one. Use template engine one.
+            resp.headers['Content-Type'] = engine_content_type
+
+            content_type = resp.headers['Content-Type']
+            if 'charset' not in content_type and (
+                        content_type.startswith('text') or content_type in ('application/xhtml+xml',
+                                                                            'application/xml',
+                                                                            'application/json')
+            ):
+                resp.content_type = content_type + '; charset=utf-8'
 
         # if it's a string return that string and skip all the stuff
         if not isinstance(response, dict):
-            if engine_name == 'json' and isinstance(response, list):
-                raise JsonEncodeError(
-                    'You may not expose with JSON a list return value because'
-                    ' it leaves your application open to CSRF attacks.')
             return result
 
-        # Save these objects as locals from the SOP to avoid expensive lookups
-        tmpl_context = tgl.tmpl_context
-
-        # If there is an identity, push it to the Pylons template context
-        tmpl_context.identity = req.environ.get('repoze.who.identity')
-
         # Setup the template namespace, removing anything that the user
         # has marked to be excluded.
         namespace = response
@@ -299,22 +261,22 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         req = context.request
 
         validation_status = req.validation
-        validation_status['exception'] = exception
+        validation_status.exception = exception
 
         if isinstance(exception, _Tw2ValidationError):
-            #Fetch all the children and grandchildren of a widget
+            # Fetch all the children and grandchildren of a widget
             widget = exception.widget
             widget_children = _navigate_tw2form_children(widget.child)
 
             errors = dict((child.compound_key, child.error_msg) for child in widget_children)
-            validation_status['errors'] = errors
-            validation_status['values'] = widget.child.value
+            validation_status.errors = errors
+            validation_status.values = widget.child.value
         elif isinstance(exception, TGValidationError):
-            validation_status['errors'] = exception.error_dict
-            validation_status['values'] = exception.value
+            validation_status.errors = exception.error_dict
+            validation_status.values = exception.value
         else:
             # Most Invalid objects come back with a list of errors in the format:
-            #"fieldname1: error\nfieldname2: error"
+            # "fieldname1: error\nfieldname2: error"
             error_list = exception.__str__().split('\n')
             for error in error_list:
                 field_value = list(map(strip_string, error.split(':', 1)))
@@ -322,21 +284,21 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
                 #if the error has no field associated with it,
                 #return the error as a global form error
                 if len(field_value) == 1:
-                    validation_status['errors']['_the_form'] = field_value[0]
+                    validation_status.errors['_the_form'] = field_value[0]
                     continue
 
-                validation_status['errors'][field_value[0]] = field_value[1]
+                validation_status.errors[field_value[0]] = field_value[1]
 
-            validation_status['values'] = getattr(exception, 'value', {})
+            validation_status.values = getattr(exception, 'value', {})
 
-        deco = controller.decoration
-
-        error_handler = deco.validation.error_handler
+        # Get the error handler associated to the current validation status.
+        error_handler = validation_status.error_handler
+        chain_validation = validation_status.chain_validation
         if error_handler is None:
             error_handler = default_im_func(controller)
+            chain_validation = False
 
-        validation_status['error_handler'] = error_handler
-        return im_self(controller), error_handler
+        return im_self(controller), error_handler, chain_validation
 
     @classmethod
     def _handle_validation_errors(cls, controller, remainder, params, exception, tgl=None):
@@ -346,20 +308,17 @@ class DecoratedController(with_metaclass(_DecoratedControllerMeta, object)):
         this is not used by TurboGears itself and is mostly provided
         for backward compatibility.
         """
-        if tgl is None: #pragma: no cover
-            #compatibility with old code that didn't pass request locals explicitly
+        warnings.warn("DecoratedController._handle_validation_errors is deprecated and will be removed",
+                      DeprecationWarning)
+
+        if tgl is None:  # pragma: no cover
+            # compatibility with old code that didn't pass request locals explicitly
             tgl = tg.request.environ['tg.locals']
 
-        obj, error_handler = cls._process_validation_errors(controller, remainder, params,
-                                                            exception, tgl)
+        obj, error_handler,_ = cls._process_validation_errors(controller, remainder, params,
+                                                              exception, tgl)
         return error_handler, error_handler(obj, *remainder, **dict(params))
 
-    def _initialize_validation_context(self, context):
-        context.request.validation = Bunch(errors={},
-                                           values={},
-                                           exception=None,
-                                           error_handler=None)
-
     def _check_security(self):
         requirement = getattr(self, 'allow_only', None)
         if requirement is None:
