@@ -15,15 +15,15 @@ This module also contains the standard ObjectDispatch
 class which provides the ordinary TurboGears mechanism.
 
 """
-import tg, sys
+import tg
 from webob.exc import HTTPException
 from tg._compat import unicode_text
-from tg.i18n import setup_i18n
 from tg.decorators import cached_property
 from crank.dispatchstate import DispatchState
 from tg.request_local import WebObResponse
 import mimetypes as default_mimetypes
 import weakref
+from ..wsgiapp import TGApp
 
 
 def dispatched_controller():
@@ -38,26 +38,23 @@ class CoreDispatcher(object):
     _use_lax_params = True
     _use_index_fallback = False
 
-    def _get_dispatchable(self, thread_locals, url_path):
+    def _get_dispatchable(self, context, url_path):
         """
-        Returns a tuple (controller, remainder, params)
+        Returns a :class:`DispatchState` instance.
 
-        :Parameters:
-          url
-            url as string
+        :param context: The Request context.
+        :param url_path: The url to dispatch.
         """
-        req = thread_locals.request
-        conf = thread_locals.config
+        req = context.request
+        conf = context.config
         
         enable_request_extensions = not conf.get('disable_request_extensions', False)
         dispatch_path_translator = conf.get('dispatch_path_translator', True)
 
-        params = req.args_params
-        state = DispatchState(weakref.proxy(req), self, params, url_path.split('/'),
+        state = DispatchState(weakref.proxy(req), self, req.args_params, url_path.split('/'),
                               conf.get('ignore_parameters', []),
                               strip_extension=enable_request_extensions,
                               path_translator=dispatch_path_translator)
-        url_path = state.path  # Get back url_path as crank performs some cleaning
 
         if enable_request_extensions:
             try:
@@ -72,18 +69,18 @@ class CoreDispatcher(object):
                 req._fast_setattr('_response_type', mime_type)
             req._fast_setattr('_response_ext', ext)
 
-        state = state.controller._dispatch(state, url_path)
+        state = state.resolve()
 
-        #save the controller state for possible use within the controller methods
+        # Save the dispatch state for possible use within the controller methods
         req._fast_setattr('_controller_state', state)
 
         if conf.get('enable_routing_args', False):
-            state.routing_args.update(params)
-            if hasattr(state.dispatcher, '_setup_wsgiorg_routing_args'):
-                state.dispatcher._setup_wsgiorg_routing_args(url_path, state.remainder,
-                                                             state.routing_args)
+            state.routing_args.update(state.params)
+            if hasattr(state.root_dispatcher, '_setup_wsgiorg_routing_args'):
+                state.root_dispatcher._setup_wsgiorg_routing_args(state.path, state.remainder,
+                                                                  state.routing_args)
 
-        return state, params
+        return state
 
     def _enter_controller(self, state, remainder):
         if hasattr(state.controller, '_visit'):
@@ -97,34 +94,23 @@ class CoreDispatcher(object):
         execution.
         """
         py_request = context.request
-        py_config = context.config
 
-        if py_config.get('i18n_enabled', True):
-            setup_i18n(context)
-
-        state, params = self._get_dispatchable(context, py_request.quoted_path_info)
-        func, controller, remainder = state.method, state.controller, state.remainder
+        state = self._get_dispatchable(context, py_request.quoted_path_info)
+        controller, action = state.controller, state.action
+        params, remainder = state.params, state.remainder
 
         if hasattr(controller, '_before'):
             controller._before(*remainder, **params)
 
         self._setup_wsgi_script_name(state.path, remainder, params)
 
-        r = self._call(func, params, remainder=remainder, context=context)
+        r = self._call(action, params, remainder=remainder, context=context)
 
         if hasattr(controller, '_after'):
             controller._after(*remainder, **params)
 
         return r
 
-    def routes_placeholder(self, url='/', start_response=None, **kwargs): #pragma: no cover
-        """Routes placeholder.
-
-        This function does not do anything.  It is a placeholder that allows
-        Routes to accept this controller as a target for its routing.
-        """
-        pass
-
     def __call__(self, environ, context):
         py_response = context.response
 
@@ -133,23 +119,35 @@ class CoreDispatcher(object):
         except HTTPException as httpe:
             response = httpe
 
-        if isinstance(response, bytes):
+        if response is tg.response or response is py_response:
+            # Controller returned the response itself, so we need to do nothing.
+            return response
+
+        if response is None:
+            # No content
+            py_response.body = b''
+            if py_response.status_int == 200:
+                # Ensure that for missing content we return 'No Content', instead of 200 OK
+                py_response.content_type = None
+                py_response.status_int = 204
+        elif isinstance(response, bytes):
             py_response.body = response
         elif isinstance(response, unicode_text):
             if not py_response.charset:
                 py_response.charset = 'utf-8'
             py_response.text = response
         elif isinstance(response, WebObResponse):
-            py_response.content_length = response.content_length
+            # Copy eventual headers from tg.response
             for name, value in py_response.headers.items():
                 header_name = name.lower()
+                if header_name in ('content-type', 'content-length'):
+                    # Do not overwrite content related headers in returned response
+                    continue
                 if header_name == 'set-cookie':
                     response.headers.add(name, value)
                 else:
                     response.headers.setdefault(name, value)
             py_response = context.response = response
-        elif response is None:
-            pass
         else:
             py_response.app_iter = response
 
@@ -178,8 +176,8 @@ class CoreDispatcher(object):
                         return v
             return []
 
-        if 'tg.root_controller' in tg.config:
-            root_controller = tg.config['tg.root_controller']
-        else:
-            root_controller = sys.modules[tg.config['application_root_module']].RootController
+        root_controller = tg.config.get('tg.root_controller')
+        if root_controller is None:
+            root_controller = TGApp.lookup_controller(tg.config, 'root')
+
         return find_url(root_controller, self, [('/', root_controller)])
