@@ -2,15 +2,14 @@
 
 Rest controller provides a RESTful dispatch mechanism, and
 combines controller decoration for TG-Controller behavior.
-
 """
 
 from crank.restdispatcher import RestDispatcher
+from tg.decorators import expose
 
 from tg.controllers.dispatcher import CoreDispatcher
 from tg.controllers.decoratedcontroller import DecoratedController
 
-
 class RestController(DecoratedController, CoreDispatcher, RestDispatcher):
     """A Decorated Controller that dispatches in a RESTful Manner.
 
@@ -71,4 +70,16 @@ class RestController(DecoratedController, CoreDispatcher, RestDispatcher):
 
     """
 
+    @expose()
+    def options(self, *args, **kw):
+        """The OPTIONS http verb is automatically sent by the browser when doing cross origin requests.
+
+        This default method was added to prevent receiving 404 HTTP Not found error. 
+        It returns None which translates to 204 No Content for HTTP. 
+        You can use the _before hook to set up your CORS headers.
+        
+        """
+        return None
+        
+
 __all__ = ['RestController']
