@@ -5,11 +5,12 @@ URL definition and browser redirection are defined here.
 
 """
 import re
+import warnings
 from webob.exc import status_map
 
 import tg
 
-from tg._compat import string_type, url_encode, unicode_text, bytes_
+from tg._compat import string_type, url_encode, unicode_text, bytes_, byte_string
 from tg.exceptions import HTTPFound
 from tg.request_local import request, response, Response
 from tg.configuration.utils import TGConfigError
@@ -19,12 +20,14 @@ def _smart_str(s):
     """
     Returns a bytestring version of 's', encoded as specified in 'encoding'.
 
-    If strings_only is True, don't convert (some) non-string-like objects.
-
     This function was borrowed from Django.
 
     """
-    if not isinstance(s, string_type):
+    if isinstance(s, byte_string):
+        return s
+    elif isinstance(s, unicode_text):
+        return s.encode('utf-8', 'strict')
+    else:
         try:
             return bytes_(s)
         except UnicodeEncodeError:
@@ -34,10 +37,6 @@ def _smart_str(s):
                 # further exception.
                 return ' '.join([_smart_str(arg).decode('utf-8') for arg in s.args]).encode('utf-8', 'strict')
             return unicode_text(s).encode('utf-8', 'strict')
-    elif isinstance(s, unicode_text):
-        return s.encode('utf-8', 'strict')
-    else:
-        return s
 
 
 def _generate_smart_str(params):
@@ -70,13 +69,16 @@ def _build_url(environ, base_url='/', params=None):
     return base_url
 
 
-def url(base_url='/', params=None, qualified=False):
+def url(base_url='/', params=None, qualified=False, scheme=None):
     """Generate an absolute URL that's specific to this application.
 
     The URL function takes a string (base_url) and, appends the
     SCRIPT_NAME and adds parameters for all of the
     parameters passed into the params dict.
 
+    ``scheme`` can be passed in case of a ``qualified`` url to
+    create an url with the given scheme.
+
     """
     if not isinstance(base_url, string_type) and hasattr(base_url, '__iter__'):
         base_url = '/'.join(base_url)
@@ -85,6 +87,8 @@ def url(base_url='/', params=None, qualified=False):
     base_url = _build_url(req.environ, base_url, params)
     if qualified:
         base_url = req.host_url + base_url
+        if scheme is not None:
+            base_url = scheme + base_url[len(req.scheme):]
 
     return base_url
 
@@ -95,15 +99,16 @@ class LazyUrl(object):
     only when you try to display it as a string.
     """
 
-    def __init__(self, base_url, params=None):
+    def __init__(self, base_url, params=None, **kwargs):
         self.base_url = base_url
         self.params = params
+        self.kwargs = kwargs
         self._decoded = None
 
     @property
     def _id(self):
         if self._decoded == None:
-            self._decoded = url(self.base_url, params=self.params)
+            self._decoded = url(self.base_url, params=self.params, **self.kwargs)
         return self._decoded
 
     @property
@@ -131,14 +136,14 @@ class LazyUrl(object):
     def startswith(self, *args, **kw):
         return self._id.startswith(*args, **kw)
 
-    def format(self, other):
-        return self._id.format(other)
+    def format(self, *args, **kwargs):
+        return self._id.format(*args, **kwargs)
 
     def __json__(self):
         return str(self)
 
 
-def lurl(base_url=None, params=None):
+def lurl(base_url=None, params=None, **kwargs):
     """
     Like tg.url but is lazily evaluated.
 
@@ -150,10 +155,10 @@ def lurl(base_url=None, params=None):
     this demands the url resolution to when it is
     displayed for the first time.
     """
-    return LazyUrl(base_url, params)
+    return LazyUrl(base_url, params, **kwargs)
 
 
-def redirect(base_url='/', params={}, redirect_with=HTTPFound, **kwargs):
+def redirect(base_url='/', params=None, redirect_with=HTTPFound, scheme=None):
     """Generate an HTTP redirect.
 
     The function raises an exception internally,
@@ -165,15 +170,14 @@ def redirect(base_url='/', params={}, redirect_with=HTTPFound, **kwargs):
     second request.
     """
 
-    if kwargs:
-        params = params.copy()
-        params.update(kwargs)
+    if params is None:
+        params = {}
 
-    new_url = url(base_url, params=params)
+    new_url = url(base_url, params=params, scheme=scheme)
     raise redirect_with(location=new_url)
 
 
-IF_NONE_MATCH = re.compile('(?:W/)?(?:"([^"]*)",?\s*)')
+IF_NONE_MATCH = re.compile(r'(?:W/)?(?:"([^"]*)",?\s*)')
 def etag_cache(key=None):
     """Use the HTTP Entity Tag cache for Browser side caching
 
@@ -185,7 +189,7 @@ def etag_cache(key=None):
     """
     if_none_matches = IF_NONE_MATCH.findall(tg.request.environ.get('HTTP_IF_NONE_MATCH', ''))
     response = tg.response._current_obj()
-    response.headers['ETag'] = '"%s"' % key
+    response.etag = key
     if str(key) in if_none_matches:
         response.headers.pop('Content-Type', None)
         response.headers.pop('Cache-Control', None)
@@ -240,7 +244,7 @@ def abort(status_code=None, detail="", headers=None, comment=None,
     def _abortion(*args, **kwargs):
         if passthrough:
             tg.request.environ['tg.status_code_redirect'] = False
-            tg.request.environ['tg.skip_auth_challenge'] = False
+            tg.request.environ['tg.skip_auth_challenge'] = True
         raise exc
 
     if error_handler is False:
@@ -305,8 +309,10 @@ def validation_errors_response(*args, **kwargs):
 
     """
     req = request._current_obj()
-    errors = dict(((str(key), str(error)) for key, error in req.validation.errors.items()))
-    values = req.validation['values']
+    errors = dict(
+        (unicode_text(key), unicode_text(error)) for key, error in req.validation.errors.items()
+    )
+    values = req.validation.values
     try:
         return Response(status=412, json_body={'errors': errors,
                                                'values': values})
