@@ -9,294 +9,29 @@ needed to support these decorators.
 
 """
 import copy
-import warnings
 import time
 from functools import partial
-from webob.exc import HTTPUnauthorized, HTTPMethodNotAllowed, HTTPMovedPermanently
+from .exceptions import HTTPUnauthorized, HTTPMethodNotAllowed, HTTPMovedPermanently
 from tg.support import NoDefault
 from tg.support.paginate import Page
 from tg.configuration import config
-from tg.configuration.app_config import _DeprecatedControllerWrapper, call_controller
 from tg.controllers.util import abort, redirect
 from tg import tmpl_context, request, response
 from tg.util import Bunch
 from tg.configuration.sqla.balanced_session import force_request_engine
 from tg.flash import flash
-from tg.caching import beaker_cache, cached_property, _cached_call
+from tg.caching import beaker_cache, cached_property, _cached_call, create_cache_key
 from tg.predicates import NotAuthorizedError
 from tg._compat import default_im_func, unicode_text
-from webob.acceptparse import Accept
-from tg.configuration import milestones
-import tg
+
+from .controllers.decoration import Decoration
+from .validation import _ValidationIntent
+
 
 import logging
 log = logging.getLogger(__name__)
 
 
-def _decorated_controller_caller(tg_config, controller, remainder, params):
-    try:
-        application_controller_caller = tg_config['controller_caller']
-    except KeyError:
-        application_controller_caller = call_controller
-
-    return application_controller_caller(tg_config, controller, remainder, params)
-
-
-class Decoration(object):
-    """ Simple class to support 'simple registration' type decorators
-    """
-    def __init__(self, controller):
-        self.controller = controller
-        self.controller_caller = _decorated_controller_caller
-        self._expositions = []
-        self.engines = {}
-        self.engines_keys = []
-        self.default_engine = None
-        self.custom_engines = {}
-        self.render_custom_format = None
-        self.validation = None
-        self.inherit = False
-        self.requirements = []
-        self.hooks = dict(before_validate=[],
-                          before_call=[],
-                          before_render=[],
-                          after_render=[])
-
-    def __repr__(self): # pragma: no cover
-        return '<Decoration %s for %r>' % (id(self), self.controller)
-
-    @classmethod
-    def get_decoration(cls, func):
-        try:
-            dec = func.decoration
-        except:
-            dec = func.decoration = cls(func)
-        return dec
-
-    def _register_exposition(self, exposition, inherit=False, before=False):
-        """Register an exposition for later application"""
-
-        # We need to store a reference to the exposition
-        # so that we can merge them when inheritance is performed
-        if before:
-            self._expositions.insert(0, exposition)
-        else:
-            self._expositions.append(exposition)
-
-        if inherit:
-            # if at least one exposition is in inherit mode
-            # all of them must inherit
-            self.inherit = True
-
-        milestones.renderers_ready.register(self._resolve_expositions)
-
-    def _resolve_expositions(self):
-        """Applies all the registered expositions"""
-        while True:
-            try:
-                exposition = self._expositions.pop(0)
-                exposition._apply()
-            except IndexError:
-                break
-
-    @property
-    def requirement(self):  # pragma: no cover
-        warnings.warn("Decoration.requirement is deprecated, "
-                      "please use 'requirements' instead", DeprecationWarning)
-        try:
-            return self.requirements[0]
-        except IndexError:
-            return None
-
-    @property
-    def exposed(self):
-        return bool(self.engines) or bool(self.custom_engines)
-
-    def merge(self, deco):
-        # This merges already registered template engines
-        self.engines = dict(tuple(deco.engines.items()) + tuple(self.engines.items()))
-        self.engines_keys = sorted(self.engines, reverse=True)
-        self.custom_engines = dict(tuple(deco.custom_engines.items()) + tuple(self.custom_engines.items()))
-
-        # This merges yet to register template engines
-        for exposition in reversed(deco._expositions):
-            self._register_exposition(exposition._clone(self.controller), before=True)
-
-        # inherit all the parent hooks
-        # parent hooks before current hooks so that they get called before
-        for hook_name, hooks in deco.hooks.items():
-            self.hooks[hook_name] = hooks + self.hooks[hook_name]
-
-        if not self.validation:
-            self.validation = deco.validation
-
-    def run_hooks(self, tgl, hook, *l, **kw):
-        warnings.warn("Decoration.run_hooks is deprecated, "
-                      "please use tg.hooks.notify instead", DeprecationWarning)
-        tg.hooks.notify(hook, args=l, kwargs=kw,
-                        controller=self.controller, context_config=tgl.config)
-
-    def register_hook(self, hook_name, func): #pragma: no cover
-        warnings.warn("Decoration.register_hook is deprecated, "
-                      "please use tg.hooks.register instead", DeprecationWarning)
-        tg.hooks.register(hook_name, func, controller=self.controller)
-
-    def register_template_engine(self,
-            content_type, engine, template, exclude_names, render_params):
-        """Registers an engine on the controller.
-
-        Multiple engines can be registered, but only one engine per
-        content_type.  If no content type is specified the engine is
-        registered at */* which is the default, and will be used
-        whenever no content type is specified.
-
-        exclude_names keeps track of a list of keys which will be
-        removed from the controller's dictionary before it is loaded
-        into the template.  This allows you to exclude some information
-        from JSONification, and other 'automatic' engines which don't
-        require a template.
-
-        render_params registers extra parameters which will be sent
-        to the rendering method.  This allows you to influence things
-        like the rendering method or the injected doctype.
-
-        """
-        default_renderer = config.get('default_renderer')
-        available_renderers = config.get('renderers', [])
-
-        if engine and not available_renderers:
-            log.warn('Renderers not registered yet while exposing template %s for engine %s, '
-                     'skipping engine availability check', template, engine)
-
-        if engine and available_renderers and engine not in available_renderers:
-            log.debug('Registering template %s for engine %s not available. Skipping it', template, engine)
-            return
-
-        content_type = content_type or '*/*'
-        if content_type in self.engines and engine != default_renderer:
-            #Avoid overwriting the default renderer when there is already a template registered
-            return
-
-        self.engines[content_type] = (engine, template, exclude_names, render_params or {})
-
-        #Avoid engine lookup if we have only one engine registered
-        if len(self.engines) == 1:
-            self.default_engine = content_type
-        else:
-            self.default_engine = None
-
-        # this is a work-around to make text/html prominent in respect
-        # to other common choices when they have the same weight for
-        # paste.util.mimeparse.best_match.
-        self.engines_keys = sorted(self.engines, reverse=True)
-
-    def register_custom_template_engine(self, custom_format,
-            content_type, engine, template, exclude_names, render_params):
-        """Registers a custom engine on the controller.
-
-        Multiple engines can be registered, but only one engine per
-        custom_format.
-
-        The engine is registered when @expose is used with the
-        custom_format parameter and controllers render using this
-        engine when the use_custom_format() function is called
-        with the corresponding custom_format.
-
-        exclude_names keeps track of a list of keys which will be
-        removed from the controller's dictionary before it is loaded
-        into the template.  This allows you to exclude some information
-        from JSONification, and other 'automatic' engines which don't
-        require a template.
-
-        render_params registers extra parameters which will be sent
-        to the rendering method.  This allows you to influence things
-        like the rendering method or the injected doctype.
-
-        """
-
-        self.custom_engines[custom_format or '"*/*"'] = (
-            content_type, engine, template, exclude_names, render_params or {})
-
-    def lookup_template_engine(self, tgl):
-        """Return the template engine data.
-
-        Provides a convenience method to get the proper engine,
-        content_type, template, and exclude_names for a particular
-        tg_format (which is pulled off of the request headers).
-
-        """
-        request = tgl.request
-        response = tgl.response
-
-        try:
-            render_custom_format = request._render_custom_format[self.controller]
-        except:
-            render_custom_format = self.render_custom_format
-
-        if render_custom_format:
-            (content_type, engine, template, exclude_names, render_params
-                ) = self.custom_engines[render_custom_format]
-        else:
-            if self.default_engine:
-                content_type = self.default_engine
-            elif self.engines:
-                if request._response_type and request._response_type in self.engines:
-                    accept_types = request._response_type
-                else:
-                    accept_types = request.headers.get('accept', '*/*')
-                content_type = Accept(accept_types).best_match(self.engines_keys, self.engines_keys[0])
-            else:
-                content_type = 'text/html'
-
-            # check for overridden content type from the controller call
-            try:
-                controller_content_type = response.headers['Content-Type']
-                # make sure we handle content_types like 'text/html; charset=utf-8'
-                content_type = controller_content_type.split(';')[0]
-            except KeyError:
-                pass
-
-            # check for overridden templates
-            try:
-                cnt_override_mapping = request._override_mapping[self.controller]
-                engine, template, exclude_names, render_params = cnt_override_mapping[content_type.split(";")[0]]
-            except (AttributeError, KeyError):
-                (engine, template, exclude_names, render_params
-                    ) = self.engines.get(content_type, (None,) * 4)
-
-        if 'charset' not in content_type and (content_type.startswith('text')
-                or  content_type in ('application/xhtml+xml',
-                        'application/xml', 'application/json')):
-            content_type += '; charset=utf-8'
-
-        return content_type, engine, template, exclude_names, render_params
-
-    def _register_hook(self, hook_name, func):
-        """Registers the specified function as a hook.
-
-        This is internal API which is used by tg.hooks, instead of
-        calling this tg.hooks.register should be used.
-
-        We now have four core hooks that can be applied by adding
-        decorators: before_validate, before_call, before_render, and
-        after_render. register_hook attaches the function to the hook
-        which get's called at the appropriate time in the request life
-        cycle.)
-        """
-        self.hooks.setdefault(hook_name, []).append(func)
-
-    def _register_requirement(self, requirement):
-        self._register_hook('before_call', requirement._check_authorization)
-        self.requirements.append(requirement)
-
-    def _register_controller_wrapper(self, wrapper):
-        try:
-            self.controller_caller = wrapper(self.controller_caller)
-        except TypeError:
-            self.controller_caller = _DeprecatedControllerWrapper(wrapper, tg.config,
-                                                                  self.controller_caller)
-
-
 class _hook_decorator(object):
     """Superclass for all the specific TG2 hook validators.
 
@@ -417,7 +152,7 @@ class expose(object):
 
     The last expose decorator example uses the custom_format parameter
     which takes an arbitrary value (in this case 'special_xml').
-    You can then use the`use_custom_format` function within the method
+    You can then use the :meth:`use_custom_format` function within the method
     to decide which of the 'custom_format' registered expose decorators
     to use to render the template.
 
@@ -527,13 +262,16 @@ def use_custom_format(controller, custom_format):
     render_custom_format[default_im_func(controller)] = custom_format
 
 
-def override_template(controller, template):
+def override_template(view, template):
     """Override the template to be used.
 
-    Use override_template in a controller in order to change the template
+    Use override_template in a controller method in order to change the template
     that will be used to render the response dictionary dynamically.
 
-    The template string passed in requires that
+    The ``view`` argument is the actual controller method for which you
+    want to replace the template.
+
+    The ``template`` string passed in requires that
     you include the template engine name, even if you're using the default.
 
     So you have to pass in a template id string like::
@@ -545,7 +283,7 @@ def override_template(controller, template):
 
     """
     try:
-        engines = controller.decoration.engines
+        engines = view.decoration.engines
     except:
         return
 
@@ -556,42 +294,33 @@ def override_template(controller, template):
             override_mapping = request._override_mapping
         except AttributeError:
             override_mapping = request._override_mapping = {}
-        override_mapping.setdefault(default_im_func(controller), {}).update({content_type: tmpl})
+        override_mapping.setdefault(default_im_func(view), {}).update({content_type: tmpl})
 
 
-class validate(object):
+class validate(_ValidationIntent):
     """Registers which validators ought to be applied.
 
     If you want to validate the contents of your form,
     you can use the ``@validate()`` decorator to register
     the validators that ought to be called.
 
-    :Parameters:
-      validators
-        Pass in a dictionary of FormEncode validators.
-        The keys should match the form field names.
-      error_handler
-        Pass in the controller method which shoudl be used
-        to handle any form errors
-      form
-        Pass in a ToscaWidget based form with validators
+    :param validators: A dictionary of FormEncode/TW2 validators, a :class:`tg.validation.Convert`
+                       or any callable that might throw :class:`tg.validation.TGValidationError`.
+    :param error_handler: Function or action that should be used to handle the errors.
+    :param form: A TW2 or ToscaWidgets form to validate ( to be provided instead of ``validators`` )
+    :param chain_validation: Whenever ``error_handler`` should perform validation too in
+                             case it's a controller action or not. By default it's disabled.
 
     The first positional parameter can either be a dictonary of validators,
     a FormEncode schema validator, or a callable which acts like a FormEncode
     validator.
-
     """
-    def __init__(self, validators=None, error_handler=None, form=None):
-        self.validators = None
-        if form:
-            self.validators = form
-        if validators:
-            self.validators = validators
-        self.error_handler = error_handler
+    def __init__(self, validators=None, error_handler=None, form=None, chain_validation=False):
+        super(validate, self).__init__(validators or form, error_handler, chain_validation)
 
     def __call__(self, func):
         deco = Decoration.get_decoration(func)
-        deco.validation = self
+        deco._register_validation(self)
         return func
 
 
@@ -600,7 +329,34 @@ class decode_params(object):
 
     By default the arguments are parsed in **JSON** format (which is
     currently the only supported format).
-
+    
+    This should be used like:
+        Image you are posting a payload of type ``application/json`` like:
+        
+        .. code-block:: javascript
+        
+            {
+                "ticketlist_id": 2,
+                "typology": "Xmas Discount",
+                "quantity": 3,
+                "price": 4.75
+            }
+        
+        Your controller's method will be something like
+        
+        .. code-block:: python
+        
+            @expose('json')
+            @decode_params(format='json')
+            def create(
+                self, typology=None,
+                quantity=None, price=None,
+                ticketlist_id=None, **kw
+            ):
+                print('*' * 60)
+                print('ticket', typology, quantity, price, ticketlist_id)
+                ...do stuff...
+                return dict(ticket=something)       
     """
     def __init__(self, format='json'):
         if format not in ('json', ):
@@ -610,7 +366,12 @@ class decode_params(object):
 
     def run_hook(self, remainder, params):
         if self._format == 'json' and request.content_type == 'application/json':
-            params.update(request.json_body)
+            try:
+                params.update(request.json_body)
+            except ValueError:
+                # Invalid JSON provided, nothing to decode
+                log.debug('Invalid JSON provided to decode_params')
+                pass
 
     def __call__(self, func):
         decoration = Decoration.get_decoration(func)
@@ -888,24 +649,27 @@ class with_engine(object):
     """
     Decorator to force usage of a specific database engine
     in TurboGears SQLAlchemy BalancedSession.
-    
+
     :param engine_name: 'master' or the name of one of the slaves, if is ``None``
              it will not force any specific engine.
     :param master_params: A dictionary or GET parameters that when present will force
              usage of the master node. The keys of the dictionary will be the
              name of the parameters to look for, while the values must be whenever
-             to pop the paramter from the parameters passed to the controller (True/False).
+             to pop the parameter from the parameters passed to the controller (True/False).
              If `master_params` is a list then it is converted to a dictionary where
              the keys are the entries of the list and the value is always True.
     """
 
-    def __init__(self, engine_name=None, master_params={}):
+    def __init__(self, engine_name=None, master_params=None):
         self.engine_name = engine_name
 
+        if master_params is None:
+            master_params = {}
+
         if not hasattr(master_params, 'keys'):
-            self.master_params = dict((p, True) for p in master_params)
-        else:
-            self.master_params = master_params
+            master_params = dict((p, True) for p in master_params)
+
+        self.master_params = master_params
 
     def before_validate(self, remainder, params):
         force_request_engine(self.engine_name)
@@ -927,19 +691,23 @@ class with_engine(object):
 
 
 class cached(object):
-    """
-    Decorator to cache the controller, if you also want to cache
-    template remember to return ``tg_cache`` option from the controller.
+    """Decorator to cache the controller.
+
+     The namespace and cache key used to cache the controller are available
+     as ``request.caching.namespace`` and ``request.caching.key``.
+     This only caches the controller, not the template, validation or the hooks associated
+     to the controller. If you also want to cache template remember to return
+     ``tg_cache`` option with the same cache key from the controller.
 
     The following parameters are accepted:
 
     ``key`` - Specifies the controller parameters used to generate the cache key.
-        NoDefault - Uses function name and all request parameters as the key (default)
-        
+        NoDefault - Uses function name and parameters (excluding args) as the key (default)
+
         None - No variable key, uses only function name as key
-        
+
         string - Use function name and only "key" parameter
-        
+
         list - Use function name and all parameters listed
     ``expire``
         Time in seconds before cache expires, or the string "never".
@@ -977,15 +745,16 @@ class cached(object):
     def __call__(self, func):
         decoration = Decoration.get_decoration(func)
 
-        def controller_wrapper(__, next_caller):
+        def controller_wrapper(next_caller):
             if self.invalidate_on_startup:
                 starttime = time.time()
             else:
                 starttime = None
 
-            def cached_call_controller(controller, remainder, params):
+            def cached_call_controller(tg_config, controller, remainder, params):
+                req = request._current_obj()
                 if self.key:
-                    key_dict = request.args_params
+                    key_dict = req.args_params
                     if self.key != NoDefault:
                         if isinstance(self.key, (list, tuple)):
                             key_dict = dict((k, key_dict[k]) for k in key_dict)
@@ -994,8 +763,12 @@ class cached(object):
                 else:
                     key_dict = {}
 
-                return _cached_call(next_caller, (controller, remainder, params), {},
-                                    key_func=func, key_dict=key_dict,
+                namespace, cache_key = create_cache_key(func, key_dict, req.controller_state.controller)
+                req._fast_setattr('caching', Bunch(namespace=namespace,
+                                                   key=cache_key))
+
+                return _cached_call(next_caller, (tg_config, controller, remainder, params), {},
+                                    namespace, cache_key,
                                     expire=self.expire, type=self.type,
                                     starttime=starttime, cache_headers=self.cache_headers,
                                     cache_response=self.cache_response,
