@@ -1,5 +1,5 @@
 """TurboGears project related information"""
-version = "2.3.4"
+version = "2.4.3"
 description = "Next generation TurboGears"
 long_description="""
 TurboGears brings together a best of breed python tools
@@ -13,7 +13,7 @@ develeopment, back database development and everything in between:
 
  * dynamic javascript powered widgets (ToscaWidgets2)
  * automatic JSON generation from your controllers
- * powerful, designer friendly XHTML based templating (Genshi)
+ * powerful, designer friendly XHTML based templating
  * object or route based URL dispatching
  * powerful Object Relational Mappers (SQLAlchemy)
 
@@ -24,8 +24,7 @@ The latest development version is available in the
     https://github.com/TurboGears
 """
 url="http://www.turbogears.org/"
-author= "Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, Michael Pedersen, Alessandro Molina, and the TurboGears community"
-email = "mark.ramm@gmail.com, alberto@toscat.net, m.pedersen@icelus.org, amol@turbogears.org"
-copyright = """Copyright 2005-2014 Kevin Dangoor,
-Alberto Valverde, Mark Ramm, Christopher Perkins and contributors"""
+author= "Alessandro Molina, Mark Ramm, Christopher Perkins, Jonathan LaCour, Rick Copland, Alberto Valverde, Michael Pedersen and the TurboGears community"
+email = "amol@turbogears.org"
+copyright = """Copyright 2005-2020 Kevin Dangoor, Alberto Valverde, Mark Ramm, Christopher Perkins, Alessandro Molina and contributors"""
 license = "MIT"
