@@ -64,11 +64,12 @@ def _get_tg_vars():
     translator = tgl.translator
     response = tgl.response
     session = tgl.session
+    helpers = conf['helpers']
 
     try:
-        h = conf['package'].lib.helpers
-    except (AttributeError, ImportError):
-        h = Bunch()
+        validation = req.validation
+    except AttributeError:
+        validation = {}
 
     # TODO: Implement user_agent and other missing features.
     tg_vars = Bunch(
@@ -80,8 +81,8 @@ def _get_tg_vars():
         identity = req.environ.get('repoze.who.identity'),
         session = session,
         locale = req.plain_languages,
-        errors = req.validation['errors'],
-        inputs = req.validation['values'],
+        errors = validation and validation.errors,
+        inputs = validation and validation.values,
         request = req,
         auth_stack_enabled = 'repoze.who.plugins' in req.environ,
         predicates = predicates)
@@ -96,14 +97,17 @@ def _get_tg_vars():
         g=app_globals,
         session=session,
         url=tg.url,
-        helpers=h,
-        h=h,
+        helpers=helpers,
+        h=helpers,
         tg=tg_vars,
         translator=translator,
         ungettext=tg.i18n.ungettext,
         _=tg.i18n.ugettext,
         N_=tg.i18n.gettext_noop)
 
+    # If there is an identity, push it to the Pylons template context
+    tmpl_context.identity = tg_vars['identity']
+
     # Allow users to provide a callable that defines extra vars to be
     # added to the template namespace
     variable_provider = conf.get('variable_provider', None)
@@ -134,7 +138,7 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
                                vars can also include the ``tg_cache`` dictionary
                                which enables template caching.
     :param str template_engine: This is the template engine name, same as
-                                specified inside AppConfig.renderers.
+                                specified inside ``tg.config['renderers']``.
     :param str template_name: This is the template to render, can be specified
                               both as a path or using dotted notation if available.
 
@@ -175,20 +179,13 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
     """
     config = tg.config._current_obj()
 
-    render_function = None
-    if template_engine is not None:
-        # the engine was defined in the @expose()
-        render_function = config['render_functions'].get(template_engine)
-
-        if render_function is None:
-            # engine was forced in @expose() but is not present in the
-            # engine list, warn developer
-            raise MissingRendererError(template_engine)
-
-    if not render_function:
-        # getting the default renderer, if no engine was defined in @expose()
+    if template_engine is None:
         template_engine = config['default_renderer']
-        render_function = config['render_functions'][template_engine]
+
+    render_function = config['render_functions'].get(template_engine)
+    if render_function is None:
+        # engine is not present in the engine list, warn developer
+        raise MissingRendererError(template_engine)
 
     if not template_vars:
         template_vars = {}
@@ -198,8 +195,7 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
     kwargs['cache_expire'] = caching_options.get('expire')
     kwargs['cache_type'] = caching_options.get('type')
 
-    for func in config.get('hooks', {}).get('before_render_call', []):
-        func(template_engine, template_name, template_vars, kwargs)
+    tg.hooks.notify('before_render_call', (template_engine, template_name, template_vars, kwargs))
 
     tg_vars = template_vars
 
@@ -211,9 +207,7 @@ def render(template_vars, template_engine=None, template_name=None, **kwargs):
 
     kwargs['result'] = render_function(template_name, tg_vars, **kwargs)
 
-    for func in config.get('hooks', {}).get('after_render_call', []):
-        func(template_engine, template_name, template_vars, kwargs)
-
+    tg.hooks.notify('after_render_call', (template_engine, template_name, template_vars, kwargs))
     return kwargs['result']
 
 
