@@ -4,6 +4,8 @@
 # Adapted to TurboGears 2.3
 from tg._compat import string_type
 from string import Template
+import logging
+
 
 def asbool(obj):
     if isinstance(obj, string_type):
@@ -45,4 +47,14 @@ def astemplate(obj):
     if not isinstance(obj, string_type):
         raise ValueError('Templates must be strings')
 
-    return Template(obj)
\ No newline at end of file
+    return Template(obj)
+
+
+def aslogger(val):
+    if isinstance(val, logging.Logger):
+        return val
+
+    if not isinstance(val, string_type):
+        raise ValueError('Logger names must be strings')
+
+    return logging.getLogger(val)
