@@ -1,19 +1,15 @@
 import os, sys, logging
 import warnings
+import inspect
 from webob.exc import HTTPNotFound
 
-log = logging.getLogger(__name__)
-
 import tg
 from tg import request_local
+from tg.configuration.utils import TGConfigError
 from tg.i18n import _get_translator
 from tg.request_local import Request, Response
 
-try: #pragma: no cover
-    import pylons
-    has_pylons = True
-except:
-    has_pylons = False
+log = logging.getLogger(__name__)
 
 
 class RequestLocals(object):
@@ -36,72 +32,53 @@ class TGApp(object):
         self.config = config = config or tg.config._current_obj()
         self.globals = config.get('tg.app_globals')
         self.package_name = config['package_name']
+        self.lang = config.get('i18n.lang')
 
         self.controller_classes = {}
         self.controller_instances = {}
 
         # Cache some options for use during requests
         self.strict_tmpl_context = self.config['tg.strict_tmpl_context']
-        self.pylons_compatible = self.config.get('tg.pylons_compatible', True)
-        self.enable_routes = self.config.get('enable_routes', False)
 
         self.resp_options = config.get('tg.response_options',
-                                       dict(content_type='text/html',
-                                            charset='utf-8',
-                                            headers={'Cache-Control': 'no-cache',
-                                                     'Pragma': 'no-cache',
-                                                     'Content-Type': None,
-                                                     'Content-Length': '0'}))
+                                       Response._DEFAULT_RESPONSE_OPTIONS)
 
-        self.wrapped_dispatch = self.dispatch
-        for wrapper in self.config.get('application_wrappers', []):
+        self.wrapped_dispatch = self._dispatch
+        for __, wrapper in self.config.get('application_wrappers', []):
             try:
-                self.wrapped_dispatch = wrapper(self.wrapped_dispatch, self.config)
+                app_wrapper = wrapper(self.wrapped_dispatch, self.config)
+                if getattr(app_wrapper, 'injected', True):
+                    # if it conforms to the ApplicationWrapper ABC inject it only
+                    # when an injected=True property is provided.
+                    self.wrapped_dispatch = app_wrapper
+
+                    # Force resolution of @cached_property, this speeds up requests
+                    # and also acts as a prevention against race conditions on the
+                    # property itself.
+                    getattr(app_wrapper, 'next_handler', None)
+
             except TypeError:
-                #backward compatibility with wrappers that didn't receive the config
+                # backward compatibility with wrappers that didn't receive the config
                 self.wrapped_dispatch = wrapper(self.wrapped_dispatch)
 
-        if 'tg.root_controller' in self.config:
+        if self.config.get('tg.root_controller') is not None:
             self.controller_instances['root'] = self.config['tg.root_controller']
 
-    def setup_pylons_compatibility(self, environ, controller): #pragma: no cover
-        """Updates environ to be backward compatible with Pylons"""
-        try:
-            environ['pylons.controller'] = controller
-            environ['pylons.pylons'] = environ['tg.locals']
-
-            self.config['pylons.app_globals'] = self.globals
-
-            pylons.request = request_local.request
-            pylons.cache = request_local.cache
-            pylons.config = request_local.config
-            pylons.app_globals = request_local.app_globals
-            pylons.session = request_local.session
-            pylons.translator = request_local.translator
-            pylons.response = request_local.response
-            pylons.tmpl_context = request_local.tmpl_context
-
-            if self.enable_routes:
-                environ['pylons.routes_dict'] = environ['tg.routes_dict']
-                pylons.url = request_local.url
-        except ImportError:
-            pass
-
     def __call__(self, environ, start_response):
+        """Serve a WSGI Request"""
         # Hide outer middlewares when crash inside application itself
         __traceback_hide__ = 'before'
 
-        testmode, context = self.setup_app_env(environ)
+        testmode, context, registry = self._setup_app_env(environ)
 
-        #Expose a path that simply registers the globals and preserves them
+        # Expose a path that simply registers the globals and preserves them
         # without doing much else
         if testmode is True and environ['PATH_INFO'] == '/_test_vars':
-            registry = environ['paste.registry']
-            registry.preserve()
+            registry.preserve(force=True)
             start_response('200 OK', [('Content-type', 'text/plain')])
             return ['DONE'.encode('utf-8')]
 
-        controller = self.resolve(environ, context)
+        controller = self._get_controller_instance('root')
         response = self.wrapped_dispatch(controller, environ, context)
 
         if testmode is True:
@@ -118,10 +95,8 @@ class TGApp(object):
             # Help Python collect ram a bit faster by removing the reference
             # cycle that the thread local objects cause
             del environ['tg.locals']
-            if has_pylons and 'pylons.pylons' in environ: #pragma: no cover
-                del environ['pylons.pylons']
 
-    def setup_app_env(self, environ):
+    def _setup_app_env(self, environ):
         """Setup Request, Response and TurboGears context objects.
 
         Is also in charge of pushing TurboGears context into the
@@ -129,21 +104,22 @@ class TGApp(object):
         the testmode is enabled or not and the TurboGears context.
         """
         conf = self.config
+        testing = False
 
         # Setup the basic global objects
         req = Request(environ)
-        req._fast_setattr('_language', conf['lang'])
+        req._fast_setattr('_language', self.lang)
         req._fast_setattr('_response_type', None)
 
         resp_options = self.resp_options
         response = Response(
             content_type=resp_options['content_type'],
             charset=resp_options['charset'],
-            headers=resp_options['headers'])
+            headers=resp_options['headers']
+        )
 
         # Setup the translator object
-        lang = conf['lang']
-        translator = _get_translator(lang, tg_config=conf)
+        translator = _get_translator(self.lang, tg_config=conf)
 
         if self.strict_tmpl_context:
             tmpl_context = TemplateContext()
@@ -151,8 +127,6 @@ class TGApp(object):
             tmpl_context = AttribSafeTemplateContext()
 
         app_globals = self.globals
-        session = environ.get('beaker.session')
-        cache = environ.get('beaker.cache')
 
         locals = RequestLocals()
         locals.response = response
@@ -161,21 +135,18 @@ class TGApp(object):
         locals.config = conf
         locals.tmpl_context = tmpl_context
         locals.translator = translator
-        locals.session = session
-        locals.cache = cache
-
-        if self.enable_routes: #pragma: no cover
-            url = environ.get('routes.url')
-            locals.url = url
+        locals.session = environ.get('beaker.session')  # Usually None, unless middleware in place
+        locals.cache = environ.get('beaker.cache')  # Usually None, unless middleware in place
 
         environ['tg.locals'] = locals
 
-        #Register Global objects
+        # Register Global objects
         registry = environ['paste.registry']
         registry.register(request_local.config, conf)
         registry.register(request_local.context, locals)
 
         if 'paste.testing_variables' in environ:
+            testing = True
             testenv = environ['paste.testing_variables']
             testenv['req'] = req
             testenv['response'] = response
@@ -184,70 +155,69 @@ class TGApp(object):
             testenv['config'] = conf
             testenv['session'] = locals.session
             testenv['cache'] = locals.cache
-            return True, locals
 
-        return False, locals
+        return testing, locals, registry
 
-    def resolve(self, environ, context):
-        """Uses dispatching information found in
-        ``environ['wsgiorg.routing_args']`` to retrieve a controller
-        name and return the controller instance from the appropriate
-        controller module.
-
-        Override this to change how the controller name is found and
-        returned.
-
-        """
-        if self.enable_routes: #pragma: no cover
-            match = environ['wsgiorg.routing_args'][1]
-            environ['tg.routes_dict'] = match
-            controller = match.get('controller')
-            if not controller:
-                return None
-        else:
-            controller = 'root'
-
-        return self.get_controller_instance(controller)
-
-    def class_name_from_module_name(self, module_name):
+    @classmethod
+    def class_name_from_module_name(cls, module_name):
         words = module_name.replace('-', '_').split('_')
         return ''.join(w.title() for w in words)
 
-    def find_controller(self, controller):
+    @classmethod
+    def lookup_controller(cls, config, controller):
         """Locates a controller by attempting to import it then grab
         the SomeController instance from the imported module.
 
         Override this to change how the controller object is found once
         the URL has been resolved.
 
+        :param dict config: The configuration options for the application,
+         usually this will be ``tg.config``.
+        :param str controller: The controller name, this will be the name
+         of the python module containing the controller.
+
         """
-        # Check to see if we've cached the class instance for this name
-        if controller in self.controller_classes:
-            return self.controller_classes[controller]
+        root_module_path = config['paths']['root']
+        base_controller_path = config['paths']['controllers']
+        if base_controller_path is None:
+            raise TGConfigError('Unable to load controllers, no controllers path configured!')
 
-        root_module_path = self.config['paths']['root']
-        base_controller_path = self.config['paths']['controllers']
-
-        #remove the part of the path we expect to be the root part (plus one '/')
+        # remove the part of the path we expect to be the root part (plus one '/')
         assert base_controller_path.startswith(root_module_path)
         controller_path = base_controller_path[len(root_module_path)+1:]
 
-        #attach the package
-        full_module_name = '.'.join([self.package_name] +
-            controller_path.split(os.sep) + controller.split('/'))
+        # attach the package
+        full_module_name = '.'.join([config['package_name']] +
+                                    controller_path.split(os.sep) +
+                                    controller.split('/'))
 
         # Hide the traceback here if the import fails (bad syntax and such)
         __traceback_hide__ = 'before_and_this'
 
         __import__(full_module_name)
         module_name = controller.split('/')[-1]
-        class_name = self.class_name_from_module_name(module_name) + 'Controller'
-        mycontroller = getattr(sys.modules[full_module_name], class_name)
+        class_name = cls.class_name_from_module_name(module_name) + 'Controller'
+        return getattr(sys.modules[full_module_name], class_name)
+
+    def find_controller(self, controller):
+        """Locates a controller for this TGApp.
+
+        This is the same af :meth:`.lookup_controller` but will reuse
+        configuration of the application and will cache results.
+
+        :param str controller: The controller name, this will be the name
+         of the python module containing the controller.
+        """
+        # Check to see if we've cached the class instance for this name
+        if controller in self.controller_classes:
+            return self.controller_classes[controller]
+
+        mycontroller = self.lookup_controller(self.config, controller)
 
         self.controller_classes[controller] = mycontroller
         return mycontroller
 
-    def get_controller_instance(self, controller):
+    def _get_controller_instance(self, controller):
         # Check to see if we've cached the instance for this name
         try:
             return self.controller_instances[controller]
@@ -255,26 +225,21 @@ class TGApp(object):
             mycontroller = self.find_controller(controller)
 
             # If it's a class, instantiate it
-            if hasattr(mycontroller, '__bases__'):
+            if inspect.isclass(mycontroller):
                 mycontroller = mycontroller()
 
             self.controller_instances[controller] = mycontroller
             return mycontroller
 
-    def dispatch(self, controller, environ, context):
+    def _dispatch(self, controller, environ, context):
         """Dispatches to a controller, the controller itself is expected
         to implement the routing system.
-
-        Override this to change how requests are dispatched to controllers.
         """
         if not controller:
             return HTTPNotFound()
 
-        #Setup pylons compatibility before calling controller
-        if has_pylons and self.pylons_compatible: #pragma: no cover
-            self.setup_pylons_compatibility(environ, controller)
-
-        # Controller is assumed to handle a WSGI call
+        # Controller is assumed to accept WSGI Environ and TG Context
+        # and return a Response object.
         return controller(environ, context)
 
 
@@ -305,7 +270,7 @@ class TemplateContext(object):
         if item in ('form_values', 'form_errors'):
             warnings.warn('tmpl_context.form_values and tmpl_context.form_errors got deprecated '
                           'use request.validation instead', DeprecationWarning)
-            return tg.request.validation[item[5:]]
+            return getattr(tg.request.validation, item[5:])
         elif item == 'controller_url':
             warnings.warn('tmpl_context.controller_url got deprecated, '
                           'use request.controller_url instead', DeprecationWarning)
